"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const manager_1 = require("./interceptor/manager");
const logger_1 = require("./logger");
const method_1 = require("./method");
const action_1 = require("./result/action");
const file_1 = require("./result/file");
const json_1 = require("./result/json");
const manager_2 = require("./result/manager");
const redirect_1 = require("./result/redirect");
const status_1 = require("./result/status");
const view_1 = require("./result/view");
const xml_1 = require("./result/xml");
const type_1 = require("./type");
const sendToWormhole = require('stream-wormhole');
const http = require('http');
[action_1.default, view_1.default, json_1.default, xml_1.default, file_1.default, redirect_1.default, status_1.default];
var DataType;
(function (DataType) {
    DataType[DataType["TEXT"] = 0] = "TEXT";
    DataType[DataType["JSON"] = 1] = "JSON";
    DataType[DataType["XML"] = 2] = "XML";
})(DataType = exports.DataType || (exports.DataType = {}));
class Action {
    static getActionMetaData(path) {
        return this.routes.get(path);
    }
    static register(config) {
        if (config === null) {
            throw new Error('注册Action必须指定配置.');
        }
        if (config.name.length === 0) {
            throw new Error('必须指定Action的名称.');
        }
        if (Action.routes.get(config.path)) {
            throw new Error('Action的路径' + config.path + '已注册.');
        }
        config.methods = config.methods || method_1.default.POST;
        if (typeof config.methods === 'string') {
            config.methods = [config.methods];
        }
        config.result = manager_2.default.create(config.result || type_1.default.action);
        config.interceptors = manager_1.InterceptorManager.create(config.interceptors);
        return (target, propertyKey, descriptor) => {
            const method = descriptor.value;
            const metadata = {
                controller: target,
                action: propertyKey,
                target: method,
                config,
            };
            for (const name of Object.getOwnPropertyNames(metadata)) {
                Object.defineProperty(metadata, name, { writable: false });
            }
            for (const name of Object.getOwnPropertyNames(metadata.config)) {
                Object.defineProperty(metadata.config, name, { writable: false });
            }
            descriptor.value = Action.wrapperAction(method, metadata);
            Action.routes.set(config.path, metadata);
        };
    }
    static registerRouter(app) {
        const { router } = app;
        Action.logger.log('=>interceptors: ', manager_1.InterceptorManager.getAll());
        Action.logger.log('=>results: ', manager_2.default.getAll());
        for (const [, metadata] of Action.routes) {
            const config = metadata.config;
            const pathName = metadata.controller.pathName.replace('controller.', '') + '.' + metadata.action;
            for (const method of config.methods) {
                const fun = router[method];
                if (typeof fun === 'function') {
                    Action.logger.log('=>route: ', config.name, config.path, pathName);
                    if (typeof config.middleware === 'function') {
                        fun.call(app, config.path, config.middleware, pathName);
                    }
                    else {
                        fun.call(app, config.path, pathName);
                    }
                }
            }
        }
    }
    static wrapperAction(method, metadata) {
        return async function () {
            const { ctx } = this;
            const params = Object.assign({}, ctx.params, ctx.request.body, ctx.request.query);
            ctx.metadata = metadata;
            ctx.params = params;
            for (const [, interceptor] of manager_1.InterceptorManager.interceptors) {
                const value = await interceptor.execute(this, metadata);
                if (!value) {
                    return;
                }
            }
            let value = await method.call(this, params);
            if (typeof value === 'undefined' || value === null) {
                value = action_1.default.success(metadata.config.name);
            }
            await Action.render(ctx, value, metadata.config.result);
        };
    }
    static async render(ctx, value, def) {
        if (ctx.body)
            return;
        let result;
        if (typeof value.render === 'function') {
            result = value;
        }
        else {
            result = def ? def : action_1.default.create(value);
        }
        await result.render(ctx, value);
    }
    static handleError(app, handler) {
        app.context.onerror = async function (err) {
            if (err == null)
                return;
            if (this.req)
                sendToWormhole(this.req);
            const headerSent = this.headerSent || !this.writable;
            if (headerSent)
                return;
            if (err.code === 'ENOENT')
                err.status = 404;
            if (typeof err.status !== 'number' || !http.STATUS_CODES[err.status]) {
                err.status = 500;
            }
            this.status = err.status;
            try {
                let value = await handler.handle(this, err);
                if (typeof value === 'undefined') {
                    value = action_1.default.fail('未处理的异常');
                }
                await Action.render(this, value);
            }
            catch (err) {
                Action.logger.error(err);
            }
            if (!this.body) {
                this.status = 500;
                this.body = 'error';
            }
            if (typeof this.body !== 'string') {
                this.body = JSON.stringify(this.body);
            }
            this.res.end(this.body);
        };
    }
}
Action.logger = new logger_1.default();
Action.dataType = DataType.JSON;
Action.routes = new Map();
exports.Action = Action;
const action = Action.register;
exports.default = action;
