"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = require("./action");
const controller_1 = require("./controller");
const enum_1 = require("./enum");
const mongo_1 = require("./parser/mongo");
const role_1 = require("./role");
const rules_1 = require("./rules");
class ApiDoc {
    static build(ctx, options) {
        const models = this.generateModel(ctx);
        const controllers = this.generateController(ctx);
        const routes = this.generateAction(controllers);
        const enums = this.generateEnums();
        return Object.assign({ routes, models, enums }, options);
    }
    static generateEnums() {
        const enums = new Array();
        for (const [, value] of enum_1.default.registry) {
            enums.push(value);
        }
        return enums;
    }
    static generateAction(controllers) {
        for (const [, route] of action_1.Action.routes) {
            const name = route.controller.pathName.replace('controller.', '');
            const controller = controllers[name];
            if (controller) {
                controller.actions.push({
                    action: route.action,
                    config: route.config,
                    role: role_1.Role.getPermissions(route.controller.constructor, route.action),
                });
            }
        }
        return controllers;
    }
    static generateController(ctx) {
        const controllers = {};
        for (const [key, value] of controller_1.ControllerAttr.attrs) {
            controllers[key] = {
                controller: value,
                actions: []
            };
        }
        return controllers;
    }
    static generateModel(ctx) {
        const models = {
            Result: rules_1.default.result,
            PageInfo: rules_1.default.pageInfo,
        };
        const { model } = ctx;
        for (const key in model) {
            if (model.hasOwnProperty(key)) {
                const element = model[key];
                models[element.modelName || key] = mongo_1.default.parse(element.schema.obj);
            }
        }
        return models;
    }
}
exports.default = ApiDoc;
