"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class FileGenerator {
    getUpperName(name) {
        return name.substring(0, 1).toUpperCase() + name.substring(1);
    }
}
exports.FileGenerator = FileGenerator;
class ControllerFileGenerator extends FileGenerator {
    generate(name) {
        return `import { Controller } from 'egg';
import action from 'ppfly/action';
import controller from 'ppfly/controller';
import ActionError from 'ppfly/error/action';
import APIError from 'ppfly/error/api';
import Model from 'ppfly/model';
import { ParamType } from 'ppfly/params';
import role from 'ppfly/role';

@role()
@controller('${name}', '${name}')
export default class ${this.getUpperName(name)}Controller extends Controller {
    @action({
        name: '测试',
        path: '/${name}/test',
        model: Model.Types.schema('${this.getUpperName(name)}'),
        params: {
            id: {
                type: ParamType.OBJECT_ID
            }
        }
    })
    public async test({ id }) {
        throw new ActionError('方法未实现.');
    }
}
`;
    }
}
exports.ControllerFileGenerator = ControllerFileGenerator;
class ModelFileGenerator extends FileGenerator {
    generate(name) {
        return `import { Application } from 'egg';
import Enum from 'ppfly/enum';
import { ParamType } from 'ppfly/params';

export default (app: Application) => {
    const mongoose = app.mongoose;
    const Schema = mongoose.Schema;

    const schema = new Schema({
        id: {
            type: Schema.Types.ObjectId,
            memo: 'ID',
            // index: true, 
            // required: true
            // $type: ParamType.FLOAT, 
            // $min: 0, 
            // $max: 0, 
            // $message: '????', 
            // $keys: Enum?, 
            // $enums: [], 
        }
    });

    return mongoose.model('${this.getUpperName(name)}', schema, '${name.toLocaleLowerCase()}');
}
        `;
    }
}
exports.ModelFileGenerator = ModelFileGenerator;
class ServiceFileGenerator extends FileGenerator {
    generate(name) {
        name = this.getUpperName(name);
        return `import { Service } from 'egg';
import ActionError from 'ppfly/error/action';
import APIError from 'ppfly/error/api';
import PagingService from 'ppfly/service/paging';
import { ObjectId, IPageInfo, IOrderInfo, Query } from 'ppfly/model';

export default class ${name}Service extends Service {
    public async find${name}ById(id: ObjectId, verify: boolean = true): Promise<I${name}>  {
        if (!id) {
            throw new ActionError('参数错误：ID不能为空！');
        }
        const obj = await this.app.model.${name}.findById(id);
        if (verify && !obj) {
            throw new ActionError('记录[' + id + ']不存在!');
        }
        return obj;
    }

    public async get${name}ListByFilter(query: Query<I${name}>, pageInfo?: IPageInfo, orderby?: IOrderInfo) {
        return PagingService.getPagingData<I${name}>(this.ctx.model.${name}, query, pageInfo, orderby);

        // return PagingService.getPagingDataByQuery<I${name}>({
        //     getDataQuery: (query) => { return this.ctx.model.${name}.find(query) },
        //     getCountQuery: (query) => { return this.ctx.model.${name}.find(query) }
        // },
        //     query,
        //     pageInfo,
        //     orderby
        // );
    }

    public async delete${name}(id: ObjectId) {
        return await this.ctx.model.${name}.deleteOne({ _id: id });
    }

    private async get${name}Count(query: Query<I${name}>) {
        return this.ctx.model.${name}.countDocuments(query);
    }
}
        `;
    }
}
exports.ServiceFileGenerator = ServiceFileGenerator;
class RulesFileGenerator extends FileGenerator {
    generate(name) {
        return `import { ParamType } from 'ppfly/params';
import Enum from 'ppfly/enum';
import Rules, { DynamicRule } from 'ppfly/rules';

export const ${this.getUpperName(name)}Rules = {
    ${name}_id: {
        ...Rules.id,
        memo: 'ID',
        message: 'ID输入不正确'
    }
}
`;
    }
}
exports.RulesFileGenerator = RulesFileGenerator;
