"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const model_1 = require("../parser/model");
class ModelGenerator {
    isProperty(prop) {
        return (prop && prop.type && prop.value
            && typeof prop.memo === 'string'
            && typeof prop.required === 'boolean');
    }
}
exports.ModelGenerator = ModelGenerator;
class ModelInterfaceGenerator extends ModelGenerator {
    getName(name) {
        return 'I' + name;
    }
    writeFields(n, obj) {
        let str = '';
        let prefix = ' '.repeat(n);
        let lines = new Array();
        for (const key in obj) {
            let prop = obj[key];
            const isNull = typeof prop !== 'object' || typeof prop.required === 'undefined' || prop.required ? '' : '?';
            const _key = `${prefix}${key}${isNull}`;
            if (this.isProperty(prop)) {
                if (prop.memo) {
                    lines.push(`${prefix}/** ${prop.memo} */`);
                }
                if (prop.type === model_1.PropertyType.Schema
                    || (Array.isArray(prop.type) && prop.type[0] === model_1.PropertyType.Schema)) {
                    prop.value = this.getName(prop.value.toString());
                }
                if (typeof prop.value === 'object') {
                    const value = this.writeFields(n + 3, prop.value);
                    prop.value = `[{\r\n${value}\r\n${prefix}}]`;
                }
                lines.push(`${_key}: ${prop.value};`);
            }
            else {
                if (typeof prop !== 'object')
                    continue;
                const value = this.writeFields(n + 3, prop);
                lines.push(`${_key}: {`);
                lines.push(value);
                lines.push(`${prefix}}`);
            }
        }
        str += lines.join('\r\n');
        return str;
    }
    generate(model) {
        const fields = this.writeFields(3, model.schema);
        return `
export interface ${this.getName(model.name)} {
${fields}
}\r\n`;
    }
}
exports.ModelInterfaceGenerator = ModelInterfaceGenerator;
