"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const params_1 = require("../params");
const model_1 = require("../parser/model");
const model_2 = require("./model");
class ModelRulesGenerator extends model_2.ModelGenerator {
    getName(name) {
        return name + 'Rules';
    }
    getType(type) {
        if (Array.isArray(type)) {
            return this.getType(type[0]);
        }
        switch (type) {
            case model_1.PropertyType.ObjectId:
                return 'ParamType.OBJECT_ID';
            case model_1.PropertyType.Number:
                return 'ParamType.NUMBER';
            case model_1.PropertyType.String:
                return 'ParamType.STRING';
            case model_1.PropertyType.Boolean:
                return 'ParamType.BOOLEAN';
            case model_1.PropertyType.Date:
                return 'ParamType.DATE';
            case model_1.PropertyType.Json:
                return 'ParamType.JSON';
            case model_1.PropertyType.Schema:
                return 'ParamType.JSON';
        }
    }
    getProps(prop) {
        const obj = {};
        obj.type = prop.$type ? prop.$type : this.getType(prop.type);
        if (!obj.type)
            return;
        obj.min = prop.$min;
        obj.max = prop.$max;
        obj.message = prop.$message ? `'${prop.$message}'` : undefined;
        obj.memo = prop.memo ? `'${prop.memo}'` : undefined;
        if (prop.required === false) {
            obj.required = prop.required;
        }
        if (Array.isArray(prop.type)) {
            obj.type = 'ParamType.ARRAY';
            if (prop.type[0] === model_1.PropertyType.Schema) {
                obj.schema = this.getName(prop.name);
            }
            else {
                obj.schema = {
                    type: this.getType(prop.type[0])
                };
            }
        }
        else if (prop.$enums) {
            obj.value = obj.type;
            obj.type = params_1.ParamType.ENUM;
            obj.values = prop.$enums;
        }
        else if (prop.$keys) {
            obj.type = 'ParamType.ENUM';
            obj.value = prop.$value || 'ParamType.NUMBER';
            obj.values = `Enum.getValues(${prop.$keys})`;
            obj.desc = `Enum.explain(${prop.$keys})`;
            obj.ref = `'${prop.$keys}'`;
        }
        return obj;
    }
    writeObject(lines, prefix, key, value) {
        lines.push(`${prefix}${key}: {`);
        lines.push(value);
        lines.push(`${prefix}},`);
    }
    _writeProps(lines, prefix, key, props) {
        for (const key in props) {
            if (!props.hasOwnProperty(key))
                continue;
            let value = props[key];
            if (typeof value === 'object') {
                this.writeProps(lines, prefix, key, value);
            }
            else if (typeof value !== 'undefined') {
                lines.push(`${prefix}${key}: ${props[key]},`);
            }
        }
    }
    writeProps(lines, prefix, key, props) {
        lines.push(`${prefix}${key}: {`);
        this._writeProps(lines, ' '.repeat(prefix.length + 3), key, props);
        lines.push(`${prefix}},`);
    }
    writeFields(n, fields) {
        let str = '';
        let prefix = ' '.repeat(n + 3);
        let lines = new Array();
        for (const key in fields) {
            if (n === 0 && key === 'meta') {
                continue;
            }
            let prop = fields[key];
            if (this.isProperty(prop)) {
                let data = this.getProps(prop);
                if (prop.type === model_1.PropertyType.Schema) {
                    data = { memo: data.memo, required: data.required };
                    lines.push(`${prefix}${key}: {`);
                    lines.push(`${prefix}${prefix}...${this.getName(prop.name)},`);
                    this._writeProps(lines, prefix + prefix, key, data);
                    lines.push(`${prefix}},`);
                    continue;
                }
                if (!data)
                    continue;
                this.writeProps(lines, prefix, key, data);
            }
            else {
                const value = this.writeFields(n + 3, prop);
                this.writeObject(lines, prefix, key, value);
            }
        }
        str += lines.join('\r\n');
        return str;
    }
    generate(model) {
        const fields = this.writeFields(0, model.schema);
        return `
export const ${this.getName(model.name)} = {
${fields}
}\r\n`;
    }
}
exports.ModelRulesGenerator = ModelRulesGenerator;
