"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Enum {
    static declare(value) {
        this.registry.set(value.name, value);
        this._registry.set(value.obj, value);
        return value;
    }
    static getSchema(obj) {
        return this._registry.get(obj);
    }
    static getSchemaByName(name) {
        return this.registry.get(name);
    }
    static getDescription(obj, value) {
        const schema = this.getSchema(obj);
        if (schema) {
            const field = schema.fields.find((p) => p.value === value);
            if (field) {
                return field.desc;
            }
        }
        return;
    }
    static getName(obj) {
        const schema = this.getSchema(obj);
        return schema ? schema.name : '';
    }
    static explain(obj) {
        const schema = this.getSchema(obj);
        if (schema) {
            const arr = schema.fields.map((p) => {
                const value = (typeof p.value === 'string' ? `'${p.value}'` : p.value);
                return `${p.key}:${value}`;
            });
            return arr.join(', ');
        }
        return '';
    }
    static getKeys(obj) {
        const schema = this.getSchema(obj);
        if (schema) {
            return schema.fields.map((p) => p.key);
        }
        return [];
    }
    static getValues(obj) {
        const schema = this.getSchema(obj);
        if (schema) {
            return schema.fields.map((p) => p.value);
        }
        return [];
    }
    static getFields(obj) {
        const schema = this.getSchema(obj);
        if (schema) {
            return schema.fields;
        }
        return [];
    }
    static isKey(key) {
        return isNaN(parseInt(key, 10));
    }
}
Enum.registry = new Map();
Enum._registry = new WeakMap();
exports.default = Enum;
