/// <reference types="node" />
export interface ObjectId {
    id: Buffer;
    toString(): string;
}
export declare type ObjectID = ObjectId;
export declare type Query<T> = {
    [P in keyof T]?: any;
};
export declare type Fields<T> = keyof T;
export declare type Nullable<T> = T | null;
export interface IOrderInfo {
    field: string;
    desc: boolean;
}
export interface IPageInfo {
    pageId?: number;
    pageSize?: number;
    dataCount?: number;
    cursor?: string | number;
}
export interface IPagingData<TData> {
    pageInfo: IPageInfo;
    data: Array<TData>;
}
export declare enum ModelType {
    Mongoose = 1
}
export interface IModelSchema {
    type: ModelType;
    schema: any;
}
export default class Model {
    static readonly schemas: Map<string, IModelSchema>;
    static readonly Types: {
        pagingData: (m: any) => string;
        array: (m: any) => string;
        schema: (m: any) => any;
    };
    static fields<T>(field: Fields<T> | Fields<T>[]): keyof T | (keyof T)[];
    static getSchema(name: string): IModelSchema | undefined;
    static declare<T>(name: string, type: ModelType, schema: T): T;
}
export declare class OrderInfo {
    static create<T>(fields: Fields<T> | Fields<T>[], desc?: boolean): IOrderInfo | IOrderInfo[];
}
export declare class PageInfo {
    static FIRST_PAGE_ID: number;
    static MAX_PAGE_SIZE: number;
    static DEFAULT_PAGE_SIZE: number;
    static create(pageSize?: number): IPageInfo;
}
export declare class PagingData {
    static create<TData>(paging: IPageInfo, data: TData[]): IPagingData<TData>;
}
