"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = require("typescript");
const base_1 = require("./base");
class EnumAstParser extends base_1.default {
    _clear(text, strs) {
        for (const str of strs) {
            text = text.replace(new RegExp(str, "gm"), '');
        }
        return text.trim();
    }
    getDeclareNode(name) {
        const nodes = this.query(`CallExpression ObjectLiteralExpression StringLiteral[text='${name}']`);
        if (nodes.length === 0) {
            return;
        }
        return nodes[0].parent.parent.parent.parent;
    }
    parse() {
        const nodes = this.query('EnumDeclaration');
        const schemas = new Array();
        for (let index = 0; index < nodes.length; index++) {
            const node = nodes[index];
            const name = node.name.escapedText;
            const m = node.getFullText().match(/\/\*[\s\S]*\*\//g);
            const desc = (m && m.length === 1) ? this._clear(m[0], ['/\\*\\*', '\\*/', '\\*', '\r', '\n']) : name;
            const schema = { name, desc, fields: [] };
            for (const obj of node.members) {
                const m = obj.getFullText().match(/\/\/.*/g);
                const desc = (m && m.length === 1) ? this._clear(m[0], ['//']) : '';
                let value;
                switch (obj.initializer.kind) {
                    case typescript_1.SyntaxKind.NumericLiteral:
                        value = obj.initializer.text;
                        break;
                    case typescript_1.SyntaxKind.PrefixUnaryExpression:
                        value = obj.initializer.getText();
                        break;
                    default:
                        value = obj.initializer.getText();
                }
                schema.fields.push({ key: obj.name.escapedText, value, desc });
            }
            schemas.push(schema);
        }
        return schemas;
    }
}
exports.default = EnumAstParser;
