"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const params_1 = require("../params");
const mongoose = require("mongoose");
class MongooseModelSchemaParser {
    static parse(model) {
        let list = {};
        for (const key in model) {
            if (model.hasOwnProperty(key)) {
                const obj = model[key];
                let schema;
                if (key === 'meta' || !obj.type) {
                    list[key] = this.parse(obj);
                    continue;
                }
                let type = 'any';
                if (obj.type === String) {
                    type = params_1.ParamType.STRING;
                }
                else if (obj.type === Number) {
                    type = 'number';
                }
                else if (obj.type === Date) {
                    type = 'date';
                }
                else if (obj.type === Array) {
                    type = 'array';
                }
                else if (Array.isArray(obj.type)) {
                    type = 'array';
                    if (obj.type.length > 0 && obj.type[0].obj) {
                        schema = this.parse(obj.type[0].obj);
                    }
                }
                else if (obj.type === Boolean) {
                    type = 'bool';
                }
                else if (obj.type === mongoose.Schema.Types.ObjectId) {
                    type = 'objectId';
                }
                else if (obj.type.obj) {
                    list[key] = this.parse(obj.type.obj);
                    continue;
                }
                list[key] = { type, memo: obj.memo, schema };
            }
        }
        return list;
    }
}
exports.default = MongooseModelSchemaParser;
