"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const params_1 = require("../params");
const model_1 = require("../parser/model");
class RulesParser {
    static getType(type) {
        switch (type) {
            case params_1.ParamType.STRING:
            case params_1.ParamType.EMAIL:
            case params_1.ParamType.URL:
            case params_1.ParamType.HASH:
            case params_1.ParamType.PHONE:
            case params_1.ParamType.JWT:
                return 'string';
            case params_1.ParamType.NUMBER:
            case params_1.ParamType.INTEGER:
            case params_1.ParamType.FLOAT:
                return 'number';
            case params_1.ParamType.DATE:
                return 'Date';
            case params_1.ParamType.BOOLEAN:
                return 'boolean';
            case params_1.ParamType.ARRAY:
                return 'any[]';
            case params_1.ParamType.ENUM:
                return 'string';
            case params_1.ParamType.JSON:
                return 'any';
            case params_1.ParamType.OBJECT_ID:
                return 'ObjectId';
            default:
                return 'any';
        }
    }
    static parseSchema(rule) {
        let type = this.getType(rule.type);
        if (rule.type === params_1.ParamType.ENUM) {
            if (rule.ref) {
                type = rule.ref;
            }
            else if (rule.value) {
                type = this.getType(rule.value);
            }
        }
        else if (rule.type === params_1.ParamType.ARRAY && rule.schema) {
            if (typeof rule.schema.type === 'string') {
                type = `${this.parseSchema(rule.schema).value}[]`;
            }
            else {
                type = this.getInterface(rule.schema);
            }
        }
        const required = (typeof rule.required === 'undefined' || rule.required) ? true : false;
        return {
            type: model_1.PropertyType.Any,
            value: type,
            memo: rule.memo || '',
            required
        };
    }
    static getInterface(params) {
        const obj = {};
        for (const key in params) {
            const value = params[key];
            if (key === 'required') {
                obj[key] = value;
                continue;
            }
            if (typeof value !== 'object') {
                continue;
            }
            if (typeof value.type === 'string') {
                obj[key] = this.parseSchema(value);
                continue;
            }
            obj[key] = this.getInterface(value);
            continue;
        }
        return obj;
    }
    static parse(name, params) {
        const obj = this.getInterface(params);
        return { name, schema: obj };
    }
}
exports.default = RulesParser;
