"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.roleSymbolKey = '__role__';
class Role {
    static decorate(permissions) {
        return (target, propertyKey, descriptor) => {
            permissions = permissions || [];
            if (!Array.isArray(permissions)) {
                permissions = [permissions];
            }
            propertyKey = propertyKey || '';
            const propertyName = exports.roleSymbolKey + propertyKey;
            target = descriptor ? target.constructor : target;
            Object.defineProperty(target, propertyName, { value: permissions, writable: false, configurable: false });
        };
    }
    static getPermissions(target, propertyKey) {
        const permissions = target[exports.roleSymbolKey + propertyKey];
        if (!permissions) {
            return target[exports.roleSymbolKey];
        }
        return permissions;
    }
    static assign(permissions) {
        const _objs = permissions;
        for (const key in permissions) {
            if (permissions.hasOwnProperty(key)) {
                _objs[key] = { key, value: permissions[key] };
            }
        }
        return permissions;
    }
    static combine(permissions) {
        const objs = {};
        for (const permission of permissions) {
            objs[permission.key] = permission;
        }
        return objs;
    }
    static merge(...permissions) {
        return Object.assign({}, ...permissions);
    }
}
exports.Role = Role;
function role(permissions) {
    return Role.decorate(permissions);
}
exports.default = role;
