"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const enum_1 = require("./enum");
const model_1 = require("./model");
const params_1 = require("./params");
const result_1 = require("./result");
exports.Rules = {
    id: {
        type: params_1.ParamType.OBJECT_ID,
        message: 'ID长度不正确',
    },
    test: {
        id: {
            type: params_1.ParamType.OBJECT_ID,
        },
        required: false
    },
    idList: {
        type: params_1.ParamType.ARRAY,
        max: 20,
        schema: {
            type: params_1.ParamType.OBJECT_ID,
            required: false,
        }
    },
    timeRange: {
        start: {
            type: params_1.ParamType.DATE,
            memo: '起始时间'
        },
        end: {
            type: params_1.ParamType.DATE,
            memo: '结束时间'
        },
        required: false
    },
    pageInfo: {
        pageId: {
            type: params_1.ParamType.NUMBER,
            min: model_1.PageInfo.FIRST_PAGE_ID,
            memo: '页面ID，从' + model_1.PageInfo.FIRST_PAGE_ID + '开始',
            required: false,
        },
        pageSize: {
            type: params_1.ParamType.NUMBER,
            max: model_1.PageInfo.MAX_PAGE_SIZE,
            memo: '页面大小，默认：' + model_1.PageInfo.DEFAULT_PAGE_SIZE,
            required: false,
        },
        dataCount: {
            type: params_1.ParamType.NUMBER,
            memo: '数据总数',
            required: false,
        },
        required: false,
    },
    orderInfo: {
        field: {
            type: params_1.ParamType.STRING,
            memo: '排序字段',
        },
        desc: {
            type: params_1.ParamType.BOOLEAN,
            memo: '是否倒序',
        },
        required: false,
    },
    result: {
        code: {
            type: params_1.ParamType.NUMBER,
            memo: `消息的CODE，成功为${result_1.Result.SUCCESS_CODE}，错误为负的整数（默认${result_1.Result.ERROR_CODE}）`
        },
        success: {
            type: params_1.ParamType.BOOLEAN,
            memo: '操作是否成功'
        },
        msg: {
            type: params_1.ParamType.STRING,
            memo: '消息内容'
        }
    }
};
exports.default = exports.Rules;
class DynamicRule {
    static orderInfo(fields) {
        return Object.assign({}, exports.Rules.orderInfo, { field: {
                type: params_1.ParamType.ENUM,
                values: fields,
                memo: '排序字段',
            } });
    }
    static enumValues(obj) {
        return {
            type: params_1.ParamType.ENUM,
            value: params_1.ParamType.NUMBER,
            values: enum_1.default.getValues(obj),
            desc: enum_1.default.explain(obj),
            ref: enum_1.default.getName(obj),
        };
    }
    static enumKeys(obj) {
        return {
            type: params_1.ParamType.ENUM,
            value: params_1.ParamType.STRING,
            values: enum_1.default.getKeys(obj),
            desc: enum_1.default.explain(obj),
            ref: enum_1.default.getName(obj),
        };
    }
}
exports.DynamicRule = DynamicRule;
