"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = require("../action");
const Redis = require("ioredis");
class RedisProvider {
    constructor(config) {
        this.SUB_KEY = '__R__cache__';
        this.client = new Redis(config);
        this.sub = new Redis(config);
    }
    subscribe(cb) {
        this.sub.subscribe(this.SUB_KEY);
        this.sub.on('message', (channel, message) => {
            if (channel === this.SUB_KEY) {
                const cmd = JSON.parse(message);
                cb(cmd);
            }
        });
    }
    async publish(cmd) {
        await this.client.publish(this.SUB_KEY, JSON.stringify(cmd));
    }
}
exports.RedisProvider = RedisProvider;
class CacheService {
    static async init(provider) {
        if (CacheService.provider) {
            return;
        }
        action_1.Action.logger.info('int cache...');
        provider.subscribe((cmd) => {
            if (cmd.type === this.CMD_CLEAR) {
                this.batchUpdate(cmd.keys).then(r => {
                    action_1.Action.logger.info('clear cache:' + r);
                });
            }
        });
        CacheService.provider = provider;
    }
    static async clear(keys) {
        keys = keys || [];
        const cmd = { type: this.CMD_CLEAR, keys };
        await this.provider.publish(cmd);
    }
    static async add(key, cache) {
        CacheService.cacheMap.set(key, cache);
    }
    static async update(key) {
        const cache = CacheService.cacheMap.get(key);
        if (cache) {
            cache.data = await cache.provider.fetchData();
        }
    }
    static async getData(key, provider) {
        const cache = CacheService.cacheMap.get(key);
        if (!cache) {
            const data = await provider.fetchData();
            await this.add(key, { data, provider });
            return data;
        }
        return cache.data;
    }
    static async batchUpdate(keys) {
        if (!keys || !keys.length) {
            const arr = new Array();
            for (const key of CacheService.cacheMap.keys()) {
                arr.push(key);
            }
            keys = arr;
        }
        for (const key in keys) {
            this.update(key);
        }
    }
}
CacheService.cacheMap = new Map();
CacheService.CMD_CLEAR = 'clear';
exports.default = CacheService;
