"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Producer = require('ali-ons').Producer;
const Consumer = require('ali-ons').Consumer;
const Message = require('ali-ons').Message;
const httpclient = require('urllib');
exports.logger = {
    info() { },
    warn() { },
    error(...args) {
        console.error(...args);
    },
    debug() { },
};
class MQService {
    static initProducer(config) {
        this.config = config;
        this.producer = new Producer({
            httpclient,
            logger: exports.logger,
            accessKeyId: config.accessKeyId,
            accessKeySecret: config.accessKeySecret,
            producerGroup: config.producerID,
        });
    }
    static initConsumer(config) {
        this.config = config;
        this.consumer = new Consumer({
            httpclient,
            logger: exports.logger,
            accessKeyId: config.accessKeyId,
            accessKeySecret: config.accessKeySecret,
            consumerGroup: config.consumerID,
        });
        this.consumer.subscribe(config.topic, '*', async (data) => {
            const msg = JSON.parse(data.body);
            const handler = this.handlers.get(msg.type);
            if (typeof handler === 'function') {
                await handler(data.msgId, msg);
            }
        });
        this.consumer.on('error', err => console.log(err));
    }
    static async push(msg) {
        if (!this.producer) {
            throw new Error('Producer 尚未初始化。');
        }
        const obj = new Message(this.config.topic, msg.type, JSON.stringify(msg));
        const result = await this.producer.send(obj);
        if (result && result.sendStatus === 'SEND_OK') {
            return true;
        }
        else {
            return false;
        }
    }
    static subscribe(type, handler) {
        this.handlers.set(type, handler);
    }
}
MQService.handlers = new Map();
exports.default = MQService;
