"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = require("../error/action");
const model_1 = require("../model");
class PagingService {
    static getOrderQuery(orderBy) {
        if (orderBy) {
            if (Array.isArray(orderBy) && orderBy.length > 0) {
                const v = {};
                for (const order of orderBy) {
                    if (order) {
                        v[order.field] = order.desc ? -1 : 1;
                    }
                }
                return v;
            }
            else {
                const order = orderBy;
                return { [order.field]: order.desc ? -1 : 1 };
            }
        }
    }
    static async getPagingDataByQuery(provider, query, pageInfo, orderBy) {
        const page = pageInfo || model_1.PageInfo.create();
        page.pageId = page.pageId || model_1.PageInfo.FIRST_PAGE_ID;
        page.pageSize = page.pageSize || model_1.PageInfo.DEFAULT_PAGE_SIZE;
        page.dataCount = page.dataCount || 0;
        page.cursor = page.cursor || undefined;
        if (page.pageSize > model_1.PageInfo.MAX_PAGE_SIZE) {
            throw new action_1.default('参数PageSize不能超过:' + model_1.PageInfo.MAX_PAGE_SIZE);
        }
        if (provider.getLastQuery && Array.isArray(orderBy)) {
            throw new Error('使用游标分页只能指定一个排序字段。');
        }
        if (page.pageId === model_1.PageInfo.FIRST_PAGE_ID
            && page.dataCount === 0) {
            page.dataCount = await provider.getCountQuery(query).countDocuments();
        }
        let _query;
        if (provider.getLastQuery) {
            query = query || {};
            orderBy = orderBy || Object.assign({}, this.DEFAULT_ORDER);
            if (page.pageId > model_1.PageInfo.FIRST_PAGE_ID && page.cursor) {
                const order = orderBy;
                query[order.field] = order.desc ? { $lt: page.cursor } : { $gt: page.cursor };
            }
            _query = provider.getLastQuery(query);
        }
        else if (provider.getDataQuery) {
            _query = provider.getDataQuery(query);
        }
        else {
            throw new Error('分页必须提供查询语句构造函数。');
        }
        if (orderBy) {
            const sort = this.getOrderQuery(orderBy);
            if (sort && Object.keys(sort).length) {
                _query.sort(sort);
            }
        }
        let list;
        if (provider.getLastQuery) {
            list = await _query.limit(page.pageSize);
            if (list && list.length > 0) {
                const order = orderBy;
                page.cursor = list[list.length - 1][order.field];
            }
        }
        else {
            list = await _query
                .skip((page.pageId - 1) * page.pageSize)
                .limit(page.pageSize);
        }
        return model_1.PagingData.create(page, list);
    }
    static async getPagingData(model, query, pageInfo, orderBy) {
        return this.getPagingDataByQuery({
            getCountQuery: (query) => { return model.find(query); },
            getDataQuery: (query) => { return model.find(query); },
        }, query, pageInfo, orderBy);
    }
}
PagingService.DEFAULT_ORDER = model_1.OrderInfo.create('_id', true);
exports.default = PagingService;
