/// <reference types="node" />
export interface IFileObject {
    path: string;
    name: string;
    size: number;
    url: string;
}
export interface ISessionObject extends IFileObject {
    id: string;
    ref_id: string;
}
export interface IStorageProvider {
    put(path: string, file: string | Buffer): Promise<IFileObject>;
    remove(path: string | string[]): Promise<void>;
}
export interface IStorageSessionManager {
    getRefFiles(ref_id: string): Promise<ISessionObject[]>;
    getExpire(): Promise<ISessionObject[]>;
    add(files: IFileObject[]): Promise<ISessionObject[]>;
    updateRef(sess_id: string[], ref_id: string): Promise<void>;
    removeRef(ref_id: string): Promise<void>;
    removeMany(sess_id: string[]): Promise<void>;
}
export declare class AliyunStorage implements IStorageProvider {
    private client;
    private host;
    constructor(host: any, client: any);
    put(path: string, file: string | Buffer): Promise<IFileObject>;
    remove(path: string | string[]): Promise<void>;
}
export default class StorageService {
    static PARAM_ID: string;
    static MAX_ID: number;
    private static storage;
    private static manager;
    static init(storage: IStorageProvider, manager: IStorageSessionManager): Promise<void>;
    static getSessionId(url: string): string;
    static appendSessionId(url: string, sessionId: string): string;
    static addFiles(files: string | Buffer | any[], path: string): Promise<IFileObject[]>;
    static verfiy(urls: string | string[]): string[];
    static updateSession(urls: string | string[], ref_id: string): Promise<number>;
    static removeFiles(ref_id: string): Promise<number>;
    static clearExpire(): Promise<number>;
}
