"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const colors = require("colors");
const fs_1 = require("fs");
const builder_1 = require("../builder");
const file_1 = require("../builder/file");
const model_1 = require("../builder/model");
const rules_1 = require("../builder/rules");
const model_2 = require("../parser/model");
const rule_1 = require("../parser/rule");
const rules_2 = require("./rules");
describe('自动构建测试', function () {
    it('创建模板代码', function () {
        const name = 'test';
        fs_1.writeFileSync(`./src/test/out/${name}.model.ts`, builder_1.default.generate(file_1.ModelFileGenerator, name), 'utf-8');
        fs_1.writeFileSync(`./src/test/out/${name}.controller.ts`, builder_1.default.generate(file_1.ControllerFileGenerator, name), 'utf-8');
        fs_1.writeFileSync(`./src/test/out/${name}.service.ts`, builder_1.default.generate(file_1.ServiceFileGenerator, name), 'utf-8');
        fs_1.writeFileSync(`./src/test/out/${name}.rule.ts`, builder_1.default.generate(file_1.RulesFileGenerator, name), 'utf-8');
    });
    it('解析模型', function () {
        const code = fs_1.readFileSync('./src/test/data/model.ts', 'utf-8');
        const parser = new model_2.default(code);
        const data = parser.parse();
        console.log(colors.green(JSON.stringify(data, null, 2)));
        assert.ok(data);
        assert.ok(parser.schemas.length === 2);
        assert.ok(parser.getInterfaceNode('IGoodsSku').length > 0);
        assert.ok(parser.getVariableNode('GoodsRules').length > 0);
        let text = '';
        for (const schema of parser.schemas) {
            text += builder_1.default.generate(model_1.ModelInterfaceGenerator, schema);
            text += builder_1.default.generate(rules_1.ModelRulesGenerator, schema);
            text += '\r\n\r\n';
        }
        fs_1.writeFileSync('./src/test/out/_model.ts', text, 'utf-8');
    });
    it('解析规则', function () {
        const schema = rule_1.default.parse('AdRules', rules_2.AdRules);
        console.log(colors.green(JSON.stringify(schema, undefined, 4)));
        const text = builder_1.default.generate(model_1.ModelInterfaceGenerator, schema);
        fs_1.writeFileSync('./src/test/out/_rules.ts', text, 'utf-8');
    });
});
