"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const parameter_1 = require("../parameter");
const params_1 = require("../params");
const validator = new parameter_1.default();
const params = {
    id: '0',
    num: '-121',
    f: '0.1',
    str: '',
    d: Date.now(),
    e: 'BIG',
    b: false,
    url: 'http://www.baidu.com',
    email: '12023023@qq.com',
    page: {
        count: '2',
        desc: true
    },
    array: ['ddd', 'd'],
    array2: [{
            id: 1212,
            name: 'ck'
        },
        {
            id: '1212',
            name: 'ck3'
        }],
    json: {
        a: 1,
        b: 2
    },
    oid: ['5f35694290679c6979769bc7', '', null, '5f35694290679c6979769bc7']
};
const rule = {
    id: {
        type: params_1.ParamType.NUMBER,
        memo: 'aaa',
    },
    num: {
        type: params_1.ParamType.INTEGER,
    },
    f: {
        type: params_1.ParamType.FLOAT,
        message: 'aaaa'
    },
    str: {
        type: params_1.ParamType.STRING,
        required: false,
        min: 2,
        max: 10
    },
    d: {
        type: params_1.ParamType.DATE,
    },
    e: {
        type: params_1.ParamType.ENUM,
        value: params_1.ParamType.STRING,
        values: ['small', 'BIG'],
    },
    b: {
        type: params_1.ParamType.BOOLEAN,
        required: false,
    },
    url: {
        type: params_1.ParamType.URL,
    },
    email: {
        type: params_1.ParamType.EMAIL,
    },
    page: {
        count: {
            type: params_1.ParamType.ENUM,
            values: [0, 1, 2, 3]
        },
        desc: {
            type: params_1.ParamType.BOOLEAN,
            required: true,
        },
        required: false
    },
    array: {
        type: params_1.ParamType.ARRAY,
        len: 2
    },
    array2: {
        type: params_1.ParamType.ARRAY,
        min: 1,
        max: 3,
        schema: {
            id: {
                type: params_1.ParamType.NUMBER,
                required: true,
            },
            name: {
                type: params_1.ParamType.STRING,
                required: true,
            },
            age: {
                type: params_1.ParamType.NUMBER,
                required: false
            }
        }
    },
    json: {
        type: params_1.ParamType.JSON,
    },
    oid: {
        type: params_1.ParamType.ARRAY,
        schema: {
            type: params_1.ParamType.OBJECT_ID,
            required: false,
        }
    }
};
describe('参数验证测试', function () {
    it('验证所有', async function () {
        let errors = await validator.validate(rule, params);
        assert(errors.length === 0);
    });
    it('验证' + params_1.ParamType.NUMBER, async function () {
        let params = { id: '10' };
        let rule = {
            id: {
                type: params_1.ParamType.NUMBER,
                memo: 'id',
            }
        };
        let errors = await validator.validate(rule, params);
        assert(params.id === 10 && errors.length === 0);
        params = { id: 0 };
        rule = {
            id: {
                type: params_1.ParamType.NUMBER,
                min: 1
            }
        };
        errors = await validator.validate(rule, params);
        console.log(errors);
        assert(params.id === 0 && errors.length === 1);
    });
    it('验证' + params_1.ParamType.BOOLEAN, async function () {
        let params = { a: 'true', b: false, c: 1000 };
        let rule = {
            a: {
                type: params_1.ParamType.BOOLEAN,
            },
            b: {
                type: params_1.ParamType.BOOLEAN,
            },
            c: {
                type: params_1.ParamType.BOOLEAN,
            },
        };
        let errors = await validator.validate(rule, params);
        console.log(errors, params);
        assert(typeof params.a === 'boolean' && params.a);
        assert(typeof params.b === 'boolean' && !params.b);
        assert(typeof params.c !== 'boolean');
        assert(errors.length === 1);
    });
});
